<?php
namespace Drupal\monobank\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

class monobankPaymentsForm extends FormBase{
    
    protected $Monobank;

    const STATUSES = [
        'new'        => 'Новий',
        'expired'    => 'Термін дії закінчився',
        'created'    => 'Оплату створено успішно, очікується оплата',
        'processing' => 'Платіж обробляється',
        'success'    => 'Успішна оплата',
        'failure'    => 'Не успішна оплата',
        'reversed'   => 'Оплата повернена',
    ];
    // ---
    public function __construct(){
        $this->Monobank = \Drupal::service('Monobank');
    }
    // ---
  	public function getFormId(){
    	return 'monobank_payments';
  	}
  	public function buildForm(array $form, FormStateInterface $form_state){
        $getCurrency = \Drupal\monobank\Form\SettingsForm::getCurrency();
        $form['#title'] = $this->Monobank->t('Payments');
        $form['filter'] = [
            '#type'         => 'details',
            '#title'        => $this->Monobank->t('Filter')
        ];
        $form['filter']['status'] = [
            '#type'         => 'select',
            '#title'        => $this->Monobank->t('Status'),
            '#options'      => $this->getOptionsStatus(),
            '#empty_option' => t('All'),
            '#default_value'=> @$_GET['status']
        ];
        $form['filter']['submit'] = [
            '#type'         => 'submit',
            '#value'        => $this->Monobank->t('Search')
        ];
        $form['table'] = [
            '#type'         => 'table',
            '#header'       => [
                [
                    'data'      => 'ID',
                    'field'     => 'l.id',
                    'sort'      => 'desc'
                ],
                [
                    'data'      => $this->Monobank->t('Created'),
                    'field'     => 'l.created'
                ],
                [
                    'data'      => 'NID',
                    'field'     => 'l.nid'
                ],
                [
                    'data'      => 'SID',
                    'field'     => 'l.sid'
                ],
                [
                    'data'      => $this->Monobank->t('User ID'),
                    'field'     => 'l.uid'
                ],
                [
                    'data'      => $this->Monobank->t('Amount'),
                    'field'     => 'l.amount'
                ],
                [
                    'data'      => $this->Monobank->t('Status'),
                    'field'     => 'l.status'
                ],
                [
                    'data'      => $this->Monobank->t('Payment time'),
                    'field'     => 'l.paytime'
                ]
            ]
        ];
        foreach ($this->getPays($form['table']['#header']) as $row){
            // ---
            $row->data = !empty($row->data) ? unserialize($row->data) : [];
            // ---
            $form['table'][$row->id]['id'] = [
                '#markup'       => $row->id
            ];
            $form['table'][$row->id]['created'] = [
                '#markup'       => date('d.m.Y H:i', $row->created)
            ];
            $form['table'][$row->id]['nid'] = [
                '#markup'       => !empty($row->nid) ? $row->nid : '- - -'
            ];
            $form['table'][$row->id]['sid'] = [
                '#markup'       => !empty($row->sid) ? $row->sid : '- - -'
            ];
            $form['table'][$row->id]['user'] = [
                '#markup'       => !empty($row->uid) ? $row->uid : '- - -'
            ];
            $form['table'][$row->id]['amount'] = [
                '#markup'       => $row->amount.' ('.($getCurrency[$row->currency] ?? '- - -').')'
            ];
            $form['table'][$row->id]['status'] = [
                '#markup'       => $this->getPayStatus($row)
            ];
            $form['table'][$row->id]['paytime'] = [
                '#markup'       => !empty($row->paytime) ? date('d.m.Y H:i', $row->paytime) : '- - -'
            ];
            if($row->status == 'new'){
                $form['table'][$row->id]['paytime']['#markup'] = Url::fromRoute('monobank.pages', [
                    'page_type'     => 'pay'
                ], [
                    'absolute'      => TRUE,
                    'query'         => [ 'pay_id' => $row->id ]
                ])->toString();
            }
        }
        $form['pager'] = [
            '#type'         => 'pager'
        ];
  		return $form;
  	}
  	public function validateForm(array &$form, FormStateInterface $form_state) {
	    parent::validateForm($form, $form_state);
	}
  	public function submitForm(array &$form, FormStateInterface $form_state){
        $query = $_GET;
        $query['status'] = $form_state->getValue('status');
        $url = Url::fromRoute('monobank.payments', [], [
            'query'     => $query
        ]);
  		$form_state->setRedirectUrl($url);
    }
    public function getPays($header){
        $query = \Drupal::database()->select('payments_monobank', 'l');
        $query->fields('l');
        if(!empty($_GET['status'])){
            $query->condition('l.status', $_GET['status']);
        }
        $table_sort = $query->extend('Drupal\Core\Database\Query\TableSortExtender')->orderByHeader($header);
        $pager = $table_sort->extend('Drupal\Core\Database\Query\PagerSelectExtender')->limit(50);
        return $pager->execute()->fetchAll();
    }
    public function getPayStatus($row){
        $status = !empty($row->status) ? $row->status : '- - -';
        if(!empty(self::STATUSES[$status])){
            $status = self::STATUSES[$status];
        }
        return $status;
    }
    public function getOptionsStatus(){
        $query = \Drupal::database()->select('payments_monobank', 'l');
        $query->fields('l', ['status', 'status']);
        $options = $query->execute()->fetchAllKeyed();
        foreach ($options as $key => $value) {
            if(!empty(self::STATUSES[$key])){
                $options[$key] = self::STATUSES[$key];
            }
        }
        return $options;
    }
}